'
' Programm gibt ber die serielle Schnittstelle eingehende Zeichen akustisch als Mosecode aus.

' Pin PB0 wird als Timer0-Ausgang OC0a verwendet, Ton wird durch Umschalten des
' Pins als Eingang / Ausgang ein- und ausgeschaltet
'
' Pin PB1 und PB2 bilden einen Software- UART (eingegebene Zeichen werden als Echo auf das Terminal zurckgesendet)
' Pin PB3 schaltet Echo um (wird PB3 auf Masse gezogen, wird das an PB1 empfangene Byte an PB2 wieder ausgegeben)
'
' Version mit Terminalbuffer: Pegelnderungen am seriellen Eingang lsen einen Interrupt aus. Die Zeichen werden
' im Hintergrund in einen FIFO geschrieben und durch die Morseroutine Zug um Zug ausgelesen.
' Die Interruptroutine bentigt leider 32 Byte SRAM. Der FIFO darf daher nur 15 Zeichen gro sein.
'
' Der FIFO ist als String- Array und nicht als Byte- Array angelegt, damit die komfortable String- Manipulationsbefehle
' verwendet werden knnen
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'
' Grundeinstelllungen:
$regfile = "ATtiny13.dat"
$crystal = 1200000
$hwstack = 40                                               ' erforderlich wegen Interrupt-Routine
$swstack = 0                                                ' wird nicht gebraucht
$framesize = 0                                              ' dito
Baud = 9600

'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

'Variablen und Konstanten dimensionieren:
'

Dim Recbyte As Byte
Dim Recchar As String * 1
Dim I As Byte                                               'Fr temporren Gebrauch als Schleifenzhler
Dim M As Byte                                               'Index des aktuellen Morsebuchstabens in unserer Tabelle
Dim L As Byte                                               ' fr niederwertigstes Bit beim bitweisen Erzeugen des Morsezeichens

Dim Pufferstring As String * 15
Dim Pufferchar As String * 1
Const Tonhoehe = 20                                         'Tonhhe
Const Punktlaenge = 100                                     'Punktlnge
Const Strichlaenge = Punktlaenge * 3                        'Strichlnge
Const Wortpause = Punktlaenge * 5                           'Lnge Wortpause


'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Initialisierung des Programms:
'
'

 ' Morseton mit Timer0 erzeugen:
'
' Timer0 zhlt das Register TCNT0 hoch. Es wird mit Register OCR0A verglichen.
' Wenn TCNT0 = OCR0A, wird Output OC0A/PB0 umgeschaltet (getoggled)
' Timer0 erzeugt so Dauerton; er wird jedoch nur an den Ausgangspin durchgeschaltet,
' wenn das Datenrichtungs- Register DDRB.0 = 1 ist. Bei 0 wird der Ton unterdrckt.
'
Config Timer0 = Counter , Prescale = 64 , Compare A = Toggle , Clear Timer = 1
Ocr0a = Tonhoehe


Acsr.acd = 0                                                'Analog-Komparator ausschalten, spart etwas Strom
Pcmsk.2 = 1                                                 ' PCINT fr Port B.2 freigeben
Gimsk.pcie = 1                                              ' PCints aktivieren
Sreg.7 = 1                                                  ' Interrupts gernerell freigeben
Portb = &B00011111                                          ' floatende Eingnge vermeiden


Open "comb.2:9600,8,n,1,inverted" For Input As #1           ' Software- UART Input an PortB.2
Open "comb.1:9600,8,n,1,inverted" For Output As #2          ' Software- UART Output an PortB.1

On Pcint0 Serielle_lesen


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -



Main:


 Do

   Gosub Zeichen_lesen

 Loop



 Zeichen_lesen:

       If Len(pufferstring) = 0 Then
       Goto Zeichen_lesen
       End If
       Pufferchar = Left(pufferstring , 1)                  ' ltestes Zeichen im Puffer
       M = Asc(pufferchar)                                  ' Ascii- Wert des untersten Zeichens im Puffer ermitteln
       M = Lookup(m , Morsetabelle)                         ' Morsezeichen aus Tabelle holen
       Gosub Morse
       Pufferstring = Mid(pufferstring , 2)                 ' das unterste Zeichen im Pufferstring entfernen

 Return



   ' Morsezeichenerzeugung:

  Morse:

      If M = 0 Then                                         ' Sonderfall: Leerzeichen;
         Waitms Wortpause
      Goto Zeichenende
      End If

      For I = 1 To 8
        If M = 1 Then                                       ' Das Byte hat nur noch den Wert 1; Zeichenende!
          Goto Zeichenende
        End If
        L = M And &B00000001                                ' niederwertigstes Bit lesen
          Ddrb.0 = 1
        If L = 1 Then
           Waitms Strichlaenge                              ' ist das Bit 1 -> dah
          Else
           Waitms Punktlaenge                               ' ist das Bit 0 -> dit
        End If
         Ddrb.0 = 0
        Waitms Punktlaenge                                  ' Pause innerhalb des Morsezeichens
        Shift M , Right , 1                                 ' Bits um eine Stelle nach rechts shiften
      Next I
     Zeichenende:
     Waitms Strichlaenge                                    ' Pause zwischen Morsezeichen

  Return



End


 Serielle_lesen:

   Pcmsk.2 = 0                                              ' Solange Bits reinkommen, PCINT sperren
   Get #1 , Recbyte                                         ' Byte lesen
   Put #2 , Recbyte                                         ' Byte auf Terminal senden (Echo)
   Recchar = Chr(recbyte)                                   ' Byte in ASCII- Zeichen wandeln
   If Len(pufferstring) < 13 Then
     Pufferstring = Pufferstring + Recchar                  ' ASCII- Zeichen ans Ende des Pufferstrings anhngen
   End If
   Pcmsk.2 = 1                                              ' PCINT wieder freigeben

 Return



'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -


Morsetabelle:

'Ascii 0 - 32 ; hier wird nur Linefeed in <kn> umgesetzt

Data 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , &B00101101 , 0 , , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0
Data 0 , 0 , 0 , 0 , 0 , 0 , 0

' Ascii 32 - 47   ->   _!"#$%&'()*+,-./
Data 0 , 0 , &B01010010 , 0 , 0 , 0 , &B00101010 , 0 , &B00101101 , &B01101101 , 0 , &B00101010 , &B01110011 , &B01100001
Data &B01101010 , &B00101001

'Ascii 48 - 57  ->  0123456789
Data &B00111111 , &B00111110 , &B00111100 , &B00111000 , &B00110000 , &B00100000 , &B00100001 , &B00100011 , &B00100111 , &B00101111

'Ascii 58 - 64  ->  :;<=>?@
Data &B01000111 , &B01110011 , &B00101101 , &B00110001 , &B01101101 , &B01001100 , 0

'Ascii 65 -  90        ABCDEFGHIJKLMNOPQRSTUVWXYZ
Data &B00000110 , &B00010001 , &B00010101 , &B00001001 , &B00000010 , &B00010100 , &B00001011 , &B00010000 , &B00000100
Data &B00011110 , &B00001101 , &B00010010 , &B00000111 , &B00000101 , &B00001111 , &B00010110 , &B00011011 , &B00001010
Data &B00001000 , &B00000011 , &B00001100 , &B00011000 , &B00001110 , &B00011001 , &B00011101 , &B00010011

'Ascii 91 - 96  ->  Muell

Data 0 , 0 , 0 , 0 , 0 , 0

' Ascii 97 - 122  abcdrfghijklmnopqrstuvwxyz

Data &B00000110 , &B00010001 , &B00010101 , &B00001001 , &B00000010 , &B00010100 , &B00001011 , &B00010000 , &B00000100
Data &B00011110 , &B00001101 , &B00010010 , &B00000111 , &B00000101 , &B00001111 , &B00010110 , &B00011011 , &B00001010
Data &B00001000 , &B00000011 , &B00001100 , &B00011000 , &B00001110 , &B00011001 , &B00011101 , &B00010011

'Ascii 123 - 127  -> Muell